/*---------------------------------------------------------------------------*\

    FILE....: hostdsp.cpp
    TYPE....: C+ Functions
    AUTHOR..: David Rowe
    DATE....: 13/5/02

    This modules simulates the "DSP" for host processing cards.

\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\

         Voicetronix Voice Processing Board (VPB) Software

         Copyright (C) 1999-2002 Voicetronix www.voicetronix.com.au

         This library is free software; you can redistribute it and/or
         modify it under the terms of the GNU Lesser General Public
         License as published by the Free Software Foundation; either
         version 2.1 of the License, or (at your option) any later version.

         This library is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
         Lesser General Public License for more details.

         You should have received a copy of the GNU Lesser General Public
         License along with this library; if not, write to the Free Software
         Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	 USA

\*---------------------------------------------------------------------------*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <pthread.h>

extern "C" {
#include "alawmulaw.h"
#include "dtmf.h"
#include "toneg.h"
#include "toned.h"
}
#include "mess.h"
#include "hostdsp.h"
#include "pci.h"
#include "messages.h"
#include "generic.h"
#include "dspfifo.h"
#include "mapdev.h"
//#include "hdaport.h"

// Open Switch data

int os_base =0;

// maps port to addr, bit pairs for ring det for that port

int ringmap[] = {
  7,0,
  7,2,
  7,4,
  7,6,
  8,0,
  8,2,
  8,4,
  8,6,
  9,0,
  9,2,
  9,4,
  9,6
};

// maps port to addr, bit pairs for hook det for that port

int hookmap[] = {
  7,1,
  7,3,
  7,5,
  7,7,
  8,1,
  8,3,
  8,5,
  8,7,
  9,1,
  9,3,
  9,5,
  9,7
};

// ring detector states

#define RING_SIL  0
#define RING_RING 1

// General

#define HOSTDSP_SLEEP 5
#define RING_SLEEP    2000
#define V12PCI_PORTS  12
#define MAX_STR       256
#define BANKA         8
#define BANKB         12
#define MAX_BOARDS    12

// V12PCI (HDA) registers

#define NUM_OE        4      // 4 Output Enable registers
#define BITS_OE       8      // bits per OE register
#define OE0           2      // OE  7...0 (MSB..LSB)
#define OE1           3      // OE 15...8
#define OE2           4      // OE 23..16
#define OE3           5      // OE 31..24
#define CONTROL_REG   6      // offset of control register

#define FRAME         32    // number of multiplexed samples/frame
#define NBUF          40    // number of frames/buffer
#define BUF           (FRAME*NBUF)

// station port hook detector (and flash detector)

#define ON_HOOK_THRESH   40
#define OFF_HOOK_THRESH  50
#define OFF_HOOK_THRESH_RINGING  50
#define PORT_DET_THRESH  40

#define HOOK_ONHOOK       0
#define HOOK_OFFHOOK      1
#define HOOK_MAYBE_ONHOOK 2
#define HOOK_MAYBE_FLASH  3

#define FLASH_THRESH      750 // max length of hook flash in ms (flash detector)
#define FLASH_DEBOUNCE    150

#define ALAW_CODES 256

// flash generator (used when dialling from loop ports)

#define FS             8000
#define FLASH_ACTIVE   0
#define FLASH_INACTIVE 1

// Ring generator states
#define RINGGEN_IDLE   0
#define RINGGEN_RING   1
#define RINGGEN_PAUSE  2

#define RINGGEN_ONTIME      1000
#define RINGGEN_OFFTIME     3000
#define RINGGEN_MAX_RINGING 4

// values of ring[] used to signal ring gen state machine
#define RINGGEN_START 1
#define RINGGEN_STOP  2

/*
typedef struct {
	int           state;
	unsigned long time;
	int           one_ring;
} RINGGEN;

typedef struct {
	int           state;
	unsigned long start_time;
	unsigned long duration;
} FLASH;
*/

typedef unsigned char uchar;                                                   
static uchar    bitrev_lut[ALAW_CODES];
static int      pci;
//static int      numbd;
/* Should be per board!!! */
//static GENERIC_CRITICAL_SECTION mutex;
//static VPBREG   *v;
//static int      finish;
//static int      active_threads;
//static int      numch;
//static int      *hook_state;
//static long     *hook_time;
//static hdaport  *hdaports;
static int      oe[NUM_OE];
//static void     **dtmf_dec;
//static void     **toneg;
//static void     **toned;
//static FLASH    *flash;

//static RINGGEN  *ringgen;
//static int      *ring;
//static int      total_ringing;

int vpb_pconf[MAX_BOARDS][BANKB];


void play();

void ProcessAudio();
void SimulateDSP(void *d);
void PlayThread(void *pv);
void RecordThread(void *pv);
void DetectRing(void *d);
void DetectLdrop(void *d);
void DetectHook(void *d);
void DetectPortConfig(void *d);
void ProcessCommands(void *d);
//void OpenPorts();
void ClosePorts();
void oe_init(int cardnum);
void oe_set(int bd, int slot);
void oe_reset(int bd, int slot);
void bitrev(uchar a[], int n);                                     
//void prepare_rec_buf(char alawbuf[][NBUF], int card);
//void prepare_play_buf(char alawbuf[][NBUF], int card);
void prepare_rec_buf(char alawbuf[][NBUF], void *pv);
void prepare_play_buf(char alawbuf[][NBUF], void *pv);
void GenerateRing(void *d);
void GenerateFlash(void *d);
void toned_callback(word mess[], void * board);



/*--------------------------------------------------------------------------*\
                                                                                                                             
        FUNCTION.: OpenSwitch::OpenSwitch
        AUTHOR...: Ben Kramer
        DATE.....: xx/02/05
                                                                                                                             
        Constructor calls osHostDSPOpen()
                                                                                                                             
\*--------------------------------------------------------------------------*/
OpenSwitch::OpenSwitch(VPBREG *ivr){
        vr = ivr;
        osHostDSPOpen();
	vr->hostdsp = this;
}
                                                                                                                             
/*--------------------------------------------------------------------------*\
                                                                                                                             
        FUNCTION.: OpenSwitch::~OpenSwitch
        AUTHOR...: Ben Kramer
        DATE.....: xx/02/05
                                                                                                                             
        Destructore calls osHostDSPClose()
                                                                                                                             
\*--------------------------------------------------------------------------*/
OpenSwitch::~OpenSwitch(){
        osHostDSPClose();
}
                                                                                                                             


/*--------------------------------------------------------------------------*\

	FUNCTION.: osHostDSPOpen
	AUTHOR...: David Rowe
	DATE.....: 13/5/02

	Start host DSP processing.

\*--------------------------------------------------------------------------*/

//void osHostDSPOpen(int anumbd, VPBREG *avr) {
void OpenSwitch::osHostDSPOpen(void) {
	int   i;
	VPB_MAN_DATA	md;

	mprintf("Initialising Host DSP ...\n");
	/* Once only stuff */
	if (pci == 0){
		mprintf("Opening first OpenSwitch card, initilizing globals!\n");
		pci = pci_open();
		// set up bitrev lut
		for(i=0; i<ALAW_CODES; i++) {
			bitrev_lut[i] = i;
		}
		bitrev(bitrev_lut, ALAW_CODES);
	}
	else {
		mprintf("Opening second and other OpenSwitch cards...\n");
	}
	active_threads = 0;
	finish = 0;

	/* Create our channel specific informtation structure */
	os_chan * chans = new os_chan[vr->numch];
	vr->chans = (void *)chans;


	/*
	ringgen = new RINGGEN[vr->numch];
	flash = new FLASH[vr->numch];
	hook_state = new int[vr->numch];
	hook_time = new long[vr->numch];
	dtmf_dec = new void*[vr->numch];
	ring = new int[vr->numch];
	*/

	toneg = new void*[vr->numch];
	vr->toned = new void*[vr->numch];
	total_ringing = 0;

	for(i=0; i<vr->numch; i++) {
		mprintf("Starting channel %d ...\n",i);
		/*
		ringgen[i].state = RINGGEN_IDLE;
		ringgen[i].time = 0;
		hook_state[i] = HOOK_ONHOOK;
		hook_time[i]= 0;
		dtmf_dec[i] = dtmf_init();
		ring[i] = 0;
		flash[i].state = FLASH_INACTIVE;
		*/
		chans[i].ringgen.state = RINGGEN_IDLE;
		chans[i].ringgen.time = 0;
		chans[i].ring = 0;
		chans[i].hook_state = HOOK_ONHOOK;
		chans[i].hook_time = 0;
		chans[i].dtmf_dec = dtmf_init();
		chans[i].flash.state = FLASH_INACTIVE;

		toneg_open(&toneg[i], i );
		toned_open(&vr->toned[i], i , &toned_callback,(void *)vr);
	}

	// disable WD timer
//	for(i=0; i<numbd; i++) {
		pci_wd_reset(pci, vr->cardtypnum);
		pci_wd_en_l(pci, vr->cardtypnum);
//	}
	
	// collect Card Info Date, Rev. and SN:
	pci_ee_mandata_read(pci,vr->cardtypnum, &md);
	strcpy(vr->mdate, md.date);
	strcpy(vr->revision, md.revision);
	strcpy(vr->serial_n, md.serial_n);
	 
	oe_init(vr->cardtypnum);
	osOpenPorts();
	DetectPortConfig(this);

	GenericInitializeCriticalSection(&mutex);
	Generic_beginthread(SimulateDSP, 0, this);
	//Generic_beginthread(RingGenThread, 0, this);
	Generic_beginthread(PlayThread, 0, this);
	Generic_beginthread(RecordThread, 0, this);
	mprintf("Initialised Host DSP for OS card[%d] date[%s] rev[%s] sn[%s] \n",vr->cardtypnum,vr->mdate,vr->revision,vr->serial_n);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: HostDSPClose
	AUTHOR...: David Rowe
	DATE.....: 13/5/02

	Stop host DSP processing.

\*--------------------------------------------------------------------------*/

void OpenSwitch::osHostDSPClose(void) {
	int i=0;
	os_chan * chans;
	chans = (os_chan*)vr->chans;

	finish = 1;
	while((active_threads)&&(i<20)){
		GenericSleep(5);
		i++;
	}
	if (i>19){
		mprintf("Check that all threads are closed\n");
	}
	osClosePorts();
//	pci_close(pci);
//	delete [] hook_state; 
//	delete [] hook_time; 
//	delete [] ring; 
//	delete [] ringgen;
//	delete [] flash; 
	for(i=0; i<vr->numch; i++) {
		dtmf_close(chans[i].dtmf_dec);
		toneg_close(toneg[i]);
		toned_close(vr->toned[i]);
	}
	delete [] toneg;
	delete [] vr->toned;
	delete [] chans;
	GenericDeleteCriticalSection(&mutex);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: OpenSwitch::osOpenPorts
	AUTHOR...: David Rowe
	DATE.....: 16/5/02

	Initialises V12PCI ports.

\*--------------------------------------------------------------------------*/

void OpenSwitch::osOpenPorts() {
	int     i;
	int     bd, ch;
	/* DR 15/8/02, causes some RX resyncs if enabled	*/
	// reset all codecs
	pci_write(pci, vr->cardtypnum, CONTROL_REG, 0x80);
        pci_write(pci, vr->cardtypnum, CONTROL_REG, 0x00);
	

	hdaports = new hdaport[vr->numch];
	for(i=0; i<vr->numch; i++) {
		ch = i % V12PCI_PORTS;
		//bd = i / V12PCI_PORTS;
		bd=vr->cardtypnum;
		hdaports[i].init(pci, bd, ch);
		
		hdaports[i].set_tx_timeslot(ch);
		hdaports[i].set_rx_timeslot(ch+V12PCI_PORTS);
		oe_set(bd, ch+V12PCI_PORTS);
		hdaports[i].set_codec_reg(0x32, 199); // codec balance

		// init DSP (audio) FIFOs

		vr->rxdf[ch] = new DspFifo(DSPFIFO_UP, vr->szrxdf[ch], vr->echo_mode);
		//printf("record side writing [%02d] 0x%x\n", ch, (int)v[bd].rxdf[ch]);
		vr->txdf[ch] = new DspFifo(DSPFIFO_DOWN, vr->sztxdf[ch], vr->echo_mode);
		//printf("play side reading [%02d] 0x%x\n", ch, (int)v[bd].txdf[ch]);

		mprintf("DSP [%02d] Audio FIFOs booted OK\n",i);
	}

}

/*--------------------------------------------------------------------------*\

	FUNCTION.: osClosePorts
	AUTHOR...: David Rowe
	DATE.....: 16/5/02

	Shuts down V12PCI ports.

\*--------------------------------------------------------------------------*/

void OpenSwitch::osClosePorts() {
	int     i;
//	int     bd, ch;

	delete [] hdaports;
	for(i=0; i<vr->numch; i++) {
		//ch = i % V12PCI_PORTS;
		//bd = i / V12PCI_PORTS;
		//delete v[bd].rxdf[ch];
		//delete v[bd].txdf[ch]; 
		delete vr->rxdf[i];
		delete vr->txdf[i]; 
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: SimulateDSP
	AUTHOR...: David Rowe
	DATE.....: 13/5/02

	Simulates the DSP on host processing cards:
	- polls card for signalling events
	- audio signal I/O is handled by different threads

\*--------------------------------------------------------------------------*/

void SimulateDSP(void *d) {

	OpenSwitch*	pThis = (OpenSwitch *)d;
	VPBREG		*vr = pThis->vr;
	GenericEnterCriticalSection(&pThis->mutex);
	pThis->active_threads++;
	GenericLeaveCriticalSection(&pThis->mutex);

	while(!pThis->finish) {
		DetectRing(pThis);
		DetectLdrop(pThis);
		DetectHook(pThis);
		ProcessCommands(pThis);
		GenerateRing(pThis);
		GenerateFlash(pThis);
		GenericSleep(HOSTDSP_SLEEP);		
	}

	GenericEnterCriticalSection(&pThis->mutex);
	pThis->active_threads--;
	GenericLeaveCriticalSection(&pThis->mutex);

	// Clean up thread
	pthread_detach(pthread_self());

	mprintf("[%02d] Finishing SimulateDSP\n",vr->cardnum);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: RingGenThread
	AUTHOR...: Peter Wintulich
	DATE.....: 23/JUL/04

	Sync. the Ring generation with zero crossing of the H/W Ring Gen.

\*--------------------------------------------------------------------------*/

void RingGenThread(void *d) {

	OpenSwitch*	pThis = (OpenSwitch *)d;
	VPBREG		*vr = pThis->vr;
	GenericEnterCriticalSection(&pThis->mutex);
	pThis->active_threads++;
	GenericLeaveCriticalSection(&pThis->mutex);

	while(!pThis->finish) {
		GenerateRing(pThis);
		GenericSleep(HOSTDSP_SLEEP);		
	}

	GenericEnterCriticalSection(&pThis->mutex);
	pThis->active_threads--;
	GenericLeaveCriticalSection(&pThis->mutex);

	mprintf("[%02d] Finishing RingGenThread\n",vr->cardnum);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: PlayThread
	AUTHOR...: David Rowe
	DATE.....: 11/6/02

	Moves audio samples between the kernel mode driver FIFO and the 
	PC FIFOs.  Play side.

\*--------------------------------------------------------------------------*/

void PlayThread(void *pv)
{
	uchar		alaw;
	int		howempty, i, ch;
	int 		b=0;
	unsigned short	buf[FRAME]={};
	char		alawbuf[V12PCI_PORTS][NBUF]={};
		
	OpenSwitch*	pThis = (OpenSwitch *)pv;
	VPBREG		*vr = pThis->vr;

//	assert(numbd < MAX_BOARDS);

	// make sure no nasty frame sync pulses (0x100)
	for(i=0; i<FRAME; i++)
		buf[i] = 0;

	GenericEnterCriticalSection(&pThis->mutex);
	pThis->active_threads++;
	GenericLeaveCriticalSection(&pThis->mutex);

	mprintf("[%02d] Starting Play Thread\n",vr->cardnum);

	int fd = pci;

	for(ch=0; ch<V12PCI_PORTS; ch++) {
		vr->txdf[ch]->SetState(1);
	}
	while(!pThis->finish) {
			
		if ((howempty = pci_tx_fifo_how_empty(fd, vr->cardtypnum))){
		while(howempty > FRAME) {
			// multiplex
			for(ch=0; ch<V12PCI_PORTS; ch++) {
				alaw = alawbuf[ch][b];
				alaw = bitrev_lut[alaw];
				buf[ch+V12PCI_PORTS] = alaw;
			}
			b++;
			if (b == NBUF) {
				b = 0;
				prepare_play_buf(alawbuf, pThis);
			}
			
			pci_tx_fifo_block_write(fd, vr->cardtypnum, FRAME, buf);
			howempty = pci_tx_fifo_how_empty(fd, vr->cardtypnum);
		}
		}

		GenericSleep(5);
	}
	for(ch=0; ch<V12PCI_PORTS; ch++) {
		vr->txdf[ch]->SetState(0);
	}
	
	GenericEnterCriticalSection(&pThis->mutex);
	pThis->active_threads--;
	GenericLeaveCriticalSection(&pThis->mutex);

	// Clean up thread
	pthread_detach(pthread_self());

	mprintf("[%02d] Finishing Play Thread\n",vr->cardnum);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: prepare_play_buf
	AUTHOR...: David Rowe
	DATE.....: 8/7/02

	Reads bufs of play samples, and formats them ready for playing.
	Also iterates tone generation.

\*--------------------------------------------------------------------------*/

void prepare_play_buf(char alawbuf[][NBUF], void *pv)
{
	int   ch, ret;
	short linbuf[NBUF];
	word  mess[DSP_LTONEG];
	OpenSwitch * pThis = (OpenSwitch *)pv;
	VPBREG * vr = pThis->vr;

	// Read play fifos, add in tone, convert to alaw
	for(ch=0; ch<V12PCI_PORTS; ch++) {
		memset(linbuf, 0, sizeof(linbuf));
		vr->txdf[ch]->DSPRead( (word*)linbuf, NBUF);

		ret = toneg_play(pThis->toneg[ch], linbuf, NBUF);
		if (ret == -1) {
			mess[0] = DSP_LTONEG;
			mess[1] = DSP_TONEG;
			mess[2] = ch;
		        vr->upmess->Write(mess, mess[0]);	
		}

		alaw_encode(&alawbuf[ch][0], linbuf, NBUF);
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: RecordThread
	AUTHOR...: David Rowe
	DATE.....: 11/6/02

	Moves audio samples between the kernel mode driver FIFO and the 
	PC FIFOs.  Record side.

\*--------------------------------------------------------------------------*/

void RecordThread(void *d) {
	unsigned short	buf[FRAME]={};
	uchar		alaw;
	int		ch,howfull=0;
	int		b=0;
	char		alawbuf[V12PCI_PORTS][NBUF]={};
	int		cardnum=0;

	OpenSwitch	*pThis = (OpenSwitch *)d;
	VPBREG		*vr = pThis->vr;
	cardnum = vr->cardtypnum;

	GenericEnterCriticalSection(&pThis->mutex);
	pThis->active_threads++;
	GenericLeaveCriticalSection(&pThis->mutex);

	mprintf("[%02d] Starting Record Thread\n",vr->cardnum);

	for(ch=0; ch<V12PCI_PORTS; ch++) {
		vr->rxdf[ch]->SetState(1);
	}
	while(!pThis->finish) {

		// audio being "recorded"

		if( (howfull = pci_rx_fifo_how_full(pci, cardnum)) ){
		while (howfull > 2*FRAME) {
			// frame of multiplexed data ready
			pci_rx_fifo_block_read(pci, vr->cardtypnum, FRAME, buf);

			// do we need to sync?

			if (!(buf[0] & 0x100)) {
				mprintf("[%02d] resyncing....\n",vr->cardnum);
				while(!(buf[0] & 0x100)) {
					pci_rx_fifo_block_read(pci, vr->cardtypnum,1,buf);
				}
				pci_rx_fifo_block_read(pci,vr->cardtypnum,FRAME-1, &buf[1]);
			} 

			// demultiplex
			for(ch=0; ch<V12PCI_PORTS; ch++) {
				alaw = buf[ch] & 0xff;
				alaw = bitrev_lut[alaw];
				alawbuf[ch][b] = (char)alaw;
			}
			b++;
			if (b == NBUF) {
				b = 0;
				prepare_rec_buf(alawbuf, pThis);
			}

			howfull = pci_rx_fifo_how_full(pci, vr->cardtypnum);
		}
		}

		GenericSleep(5);
	}
	for(ch=0; ch<V12PCI_PORTS; ch++) {
		vr->rxdf[ch]->SetState(0);
	}

	GenericEnterCriticalSection(&pThis->mutex);
	pThis->active_threads--;
	GenericLeaveCriticalSection(&pThis->mutex);

	// Clean up thread
	pthread_detach(pthread_self());

	mprintf("[%02d] Finishing Record Thread\n",vr->cardnum);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: prepare_rec_buf
	AUTHOR...: David Rowe
	DATE.....: 25/6/02

	Transfers buffers of recorded samples to the driver FIFOs.  Also
	runs DTMF decoders on each channel.

\*--------------------------------------------------------------------------*/

void prepare_rec_buf(char alawbuf[][NBUF], void *pv)
{
	int		ch, digits_ret;
	short		linbuf[NBUF];
	char		key[2];
        word		mess[DSP_LDTMF];
	OpenSwitch	*pThis = (OpenSwitch *)pv;
	VPBREG		*vr=pThis->vr;
	os_chan		*chans = (os_chan*)vr->chans;
	int		keydown=0;

	// convert to linear and send to fifos
	for(ch=0; ch<V12PCI_PORTS; ch++) {
		alaw_decode(linbuf, &alawbuf[ch][0], NBUF);
		vr->rxdf[ch]->DSPWrite( (word*)linbuf, NBUF);

		// DTMF processing

		keydown = 0;
		digits_ret = dtmf_decoder(chans[ch].dtmf_dec, key, 2, linbuf, NBUF, &keydown);
		
		if (digits_ret) {
			// it should be impossible to detect > 1 digit in NBUF
			// sample buffer
			assert(digits_ret == 1);
			//mprintf("ch = %d bd = %d DTMF.....\n", ch, bd);
			mess[0] = DSP_LDTMF;
			mess[1] = DSP_DTMF;
			mess[2] = ch;
			mess[3] = key[0];
			vr->upmess->Write(mess, mess[0]);	
		}
		else if (keydown){
		//	mprintf("ch = %d bd = %d DTMF DOWN.....\n", ch, vr->cardnum);
			mess[0] = DSP_LDTMF_DOWN;
			mess[1] = DSP_DTMF_DOWN;
			mess[2] = ch;
			mess[3] = keydown;
			vr->upmess->Write(mess, mess[0]);	
		}

		// tone decoder processing
		toned_analyse(vr->toned[ch], linbuf, NBUF);
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: DetectLdrop
	AUTHOR...: Ben Kramer
	DATE.....: 29/1/03

	Based on DetectRing

\*--------------------------------------------------------------------------*/

void DetectLdrop(void *d) {
	int		i, ldrop; 
	//int		bd,ch;
        word		rmess[DSP_LDROP];
	OpenSwitch	*pThis = (OpenSwitch *)d;
	VPBREG		*vr=pThis->vr;

	for(i=0; i<vr->numch; i++) {
		//ch = i % V12PCI_PORTS;
		//bd = i / V12PCI_PORTS;
		

		//ldrop = pci_kldrop_read(pci, (unsigned short)bd, ch);
		if ( (ldrop = pci_kldrop_read(pci, (unsigned short)vr->cardtypnum, i)) ){
			if (ldrop) {
				// generate event
				rmess[0] = DSP_LDROP;
				rmess[1] = DSP_DROP;
				rmess[2] = i;
				vr->upmess->Write(rmess, rmess[0]);	
			}
		}
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: DetectRing
	AUTHOR...: David Rowe
	DATE.....: 13/5/02

	Simple ring detector:
	- looks for rising edge
	- emits ring event
	- waits for RING_SLEEP ms before sensing next ring event

	Note: this might need to be replaced by V4PCI-like ring det in
	future, implemented at kernel level to get exact timing.

\*--------------------------------------------------------------------------*/

void DetectRing(void *d) {
	int		i;
	int		ring=0; 
	//int		bd,ch;
        word		rmess[DSP_LCODEC_RING];
	OpenSwitch	*pThis = (OpenSwitch *)d;
	VPBREG		*vr=pThis->vr;

	for(i=0; i<vr->numch; i++) {
		//ch = i % V12PCI_PORTS;
		//bd = i / V12PCI_PORTS;

		ring = pci_kring_read(pci, (unsigned short)vr->cardtypnum, i);
		if (ring == 1) {
			// generate event
			rmess[0] = DSP_LCODEC_RING;
			rmess[1] = DSP_CODEC_RING;
			rmess[2] = i;
			vr->upmess->Write(rmess, rmess[0]);	
		}
		if (ring == -1) {
			// generate event
			rmess[0] = DSP_LRING_OFF;
			rmess[1] = DSP_RING_OFF;
			rmess[2] = i;
			vr->upmess->Write(rmess, rmess[0]);	
		}
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: DetectHook
	AUTHOR...: David Rowe
	DATE.....: 14/5/02

	Detects hook status for station ports.

\*--------------------------------------------------------------------------*/

void DetectHook(void *d) {
	int		i, state, next_state; 
	//int		bd, ch;
	int		running;
        word		mess[DSP_LCODEC_HKOFF];
	unsigned long	duration;
	int		off_hook_thresh;
	OpenSwitch	*pThis = (OpenSwitch *)d;
	VPBREG		*vr=pThis->vr;
	os_chan		*chans = (os_chan*)vr->chans;

	for(i=0; i<vr->numch; i++) {
		//ch = i % V12PCI_PORTS;
		//bd = i / V12PCI_PORTS;

		//running = pci_khook_read(pci, (unsigned short)bd, ch);
		running = pci_khook_read(pci, (unsigned short)vr->cardtypnum, i);
		// used to "tune" HOOK_THRESH during ring (unittest/ringstat)
		/* 
		if (i == 0)
			mprintf("running = %d\n",running);
		*/

		state = chans[i].hook_state;
		next_state = chans[i].hook_state;
		if ( chans[i].ringgen.state == RINGGEN_IDLE){
			off_hook_thresh=OFF_HOOK_THRESH;
		}
		else {
			off_hook_thresh= OFF_HOOK_THRESH_RINGING;
		}

		switch(state) {
		case HOOK_ONHOOK:
			if (running > off_hook_thresh) {
				next_state = HOOK_OFFHOOK;
				
				// stop ringing
				chans[i].ring = RINGGEN_STOP;
				pThis->hdaports[i].set_ring_off();

				pThis->hdaports[i].set_audio_on();

				// generate event
				mess[0] = DSP_LCODEC_HKOFF;
				mess[1] = DSP_CODEC_HKOFF;
				mess[2] = i;
				vr->upmess->Write(mess, mess[0]);	
			}
			break;

		case HOOK_OFFHOOK:
			if (running < ON_HOOK_THRESH) {
				next_state = HOOK_MAYBE_ONHOOK;
				chans[i].hook_time = GenerictimeGetTime();
			}
			break;

		case HOOK_MAYBE_ONHOOK:

			// check we are still on hook
			if (running < ON_HOOK_THRESH) {

				// if on hook for > max flash duration
				// must be a real on hook
				duration = GenerictimeGetTime() - chans[i].hook_time;
				if (duration > FLASH_THRESH) {
					pThis->hdaports[i].set_audio_off();
					next_state = HOOK_ONHOOK;
				        // generate event
					mess[0] = DSP_LCODEC_HKON;
					mess[1] = DSP_CODEC_HKON;
					mess[2] = i;
					vr->upmess->Write(mess, mess[0]);
				}
		        }
			else {
				// maybe a flash
				chans[i].hook_time = GenerictimeGetTime();
				next_state = HOOK_MAYBE_FLASH;
			}
			break;
		case HOOK_MAYBE_FLASH:
			duration = GenerictimeGetTime() - chans[i].hook_time;
			if (duration > FLASH_DEBOUNCE) {
				// check we are still on hook
				if (running < ON_HOOK_THRESH) {
					pThis->hdaports[i].set_audio_off();
					next_state = HOOK_ONHOOK;
					// generate event
					mess[0] = DSP_LCODEC_HKON;
					mess[1] = DSP_CODEC_HKON;
					mess[2] = i;
					vr->upmess->Write(mess, mess[0]);
				}
				else {
					// else must have been a flash
					next_state = HOOK_OFFHOOK;
					// generate event
					mess[0] = DSP_LCODEC_FLASH;
					mess[1] = DSP_CODEC_FLASH;
					mess[2] = i;
					vr->upmess->Write(mess, mess[0]);
				}
			}
			break;
		}


		chans[i].hook_state = next_state;
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: ProcessCommands
	AUTHOR...: David Rowe
	DATE.....: 17/5/02

	Reads commands sent to "DSP" and processes them.

\*--------------------------------------------------------------------------*/

void ProcessCommands(void *d) {
	int            card, read, config, lsf_ms, id; 
        word           mess[MAX_STR];

	OpenSwitch	*pThis = (OpenSwitch *)d;
	VPBREG		*vr=pThis->vr;
	os_chan		* chans = (os_chan*)vr->chans;

	card = vr->cardtypnum;
	read = vr->dnmess->Read(&mess[0], 1);	
	if (read == OK) {
		read = vr->dnmess->Read(&mess[1], mess[0]-1);	
		//printf("mess[0] = %d read = %d\n", mess[0], read);
		switch(mess[1]) {
		case PC_CODEC_FXSONHK:
			assert(mess[0] == PC_LCODEC_ONHK);
			pThis->hdaports[mess[2]].set_loop_onhook();
			break;

		case PC_CODEC_FXSOFFHK:
			assert(mess[0] == PC_LCODEC_OFFHK);

			config = pThis->hdaports[mess[2]].get_config();
			if (config == HDAPORT_STATION) {
				pThis->hdaports[mess[2]].set_loop_offhook();
			}
			break;
		case PC_CODEC_ONHK:
			assert(mess[0] == PC_LCODEC_ONHK);
			pThis->hdaports[mess[2]].set_loop_onhook();
			break;

		case PC_CODEC_OFFHK:
			assert(mess[0] == PC_LCODEC_OFFHK);

			// only allow loop ports to go offhook

			config = pThis->hdaports[mess[2]].get_config();
			if (config == HDAPORT_LOOP) {
				pThis->hdaports[mess[2]].set_loop_offhook();
			}
			break;

		case PC_STATION_OFF:
			assert(mess[0] == PC_LSTATION_OFF);
			pThis->hdaports[mess[2]].set_audio_off();
			break;

		case PC_STATION_ON:
			assert(mess[0] == PC_LSTATION_ON);
			pThis->hdaports[mess[2]].set_audio_on();
			break;

		case PC_RING_ON:
			// cadenced ring gen start
			assert(mess[0] == PC_LRING_ON);
			chans[mess[2]].ringgen.one_ring = mess[3];
			chans[mess[2]].ring = RINGGEN_START;
			break;

		case PC_RING_OFF:
			// cadenced ring gen stop
			assert(mess[0] == PC_LRING_OFF);
			chans[mess[2]].ring = RINGGEN_STOP;
			break;

		case PC_URING_OFF:
			assert(mess[0] == PC_LURING_OFF);
			pThis->hdaports[mess[2]].set_ring_off();
			break;

		case PC_URING_ON:
			assert(mess[0] == PC_LURING_ON);
			pThis->hdaports[mess[2]].set_ring_on();
//				mprintf("ring on %d\n", offset + mess[2]);
			break;

		case PC_SPI_BRIDGE:
			assert(mess[0] == PC_LSPI_BRIDGE);
			pThis->hdaports[mess[2]].set_rx_timeslot(mess[3]);
			pThis->hdaports[mess[3]].set_rx_timeslot(mess[2]);
			break;

		case PC_SPI_BRIDGE_OFF:
			assert(mess[0] == PC_LSPI_BRIDGE_OFF);
			pThis->hdaports[mess[2]].set_rx_timeslot(mess[2]+V12PCI_PORTS);
			pThis->hdaports[mess[3]].set_rx_timeslot(mess[3]+V12PCI_PORTS);
			break;

		case PC_SPI_SBRIDGE:
			assert(mess[0] == PC_LSPI_SBRIDGE);
			pThis->hdaports[mess[2]].sbridge_on((int)mess[3],(int)mess[4]);
			break;

		case PC_SPI_SBRIDGE_OFF:
			assert(mess[0] == PC_LSPI_SBRIDGE_OFF);
			pThis->hdaports[mess[2]].sbridge_off();
			break;

		case PC_CONF_JOIN:
			assert(mess[0] == PC_LCONF_JOIN);
			pThis->hdaports[mess[2]].conf_join(mess[3]);
			break;

		case PC_CONF_LEAVE:
			assert(mess[0] == PC_LCONF_LEAVE);
			pThis->hdaports[mess[2]].conf_leave();
			break;

		case PC_CODEC_RECGAIN:
			assert(mess[0] == PC_LCODEC_RECGAIN);
			pThis->hdaports[mess[2]].set_rec_gain((unsigned char)mess[3]);
			break;

		case PC_CODEC_PLAYGAIN:
			assert(mess[0] == PC_LCODEC_PLAYGAIN);
			pThis->hdaports[mess[2]].set_play_gain((unsigned char)mess[3]);
		break;

		case PC_TONEG:
			assert(mess[0] == PC_LTONEG);
			toneg_start(pThis->toneg[mess[12]], mess);
		break;

		case PC_TONEG_RESET:
			toneg_reset(pThis->toneg[mess[2]], mess);
			break;

		case PC_CODEC_BREAK:
			// timed loop break - generate flash
			lsf_ms = vr->lsf*1000/FS;
			chans[mess[2]].flash.duration = lsf_ms*mess[3];
			chans[mess[2]].flash.start_time = GenerictimeGetTime();
			pThis->hdaports[mess[2]].set_loop_onhook();
			chans[mess[2]].flash.state = FLASH_ACTIVE;
			mprintf("lsf_ms = %d, duration = %ld\n", lsf_ms, chans[mess[2]].flash.duration);
			break;

		case PC_LOOPBACK_ON:
			assert(mess[0] == PC_LLOOPBACK_ON);
			pci_loopback_on(pci, card);	
			break;

		case PC_LOOPBACK_OFF:
			assert(mess[0] == PC_LLOOPBACK_OFF);
			pci_loopback_off(pci, card);	
			break;

		case PC_HOSTECHO_ON:
			assert(mess[0] == PC_LHOSTECHO_ON);
			pci_hostecho_on(pci,((card*12)+mess[2]));	
			break;

		case PC_HOSTECHO_OFF:
			assert(mess[0] == PC_LHOSTECHO_OFF);
			pci_hostecho_off(pci, ((card*12)+mess[2]));	
			break;

		case PC_HOSTECHO_OPT:
			assert(mess[0] == PC_LHOSTECHO_OFF);
			pci_hostecho_opt(pci, mess[2]);	
			break;

		case PC_WD_ENABLE:
			assert(mess[0] == PC_LWD_ENABLE);
			if (mess[2]) {
				pci_wd_en_h(pci, card);
			} else {
				pci_wd_en_l(pci, card);
			}
				
			break;

		case PC_WD_RESET:
			assert(mess[0] == PC_LWD_RESET);
			pci_wd_reset(pci, card);	
			break;

		case PC_TONED_ST:  	
			// call progress detector messages 
			id = mess[2];
			//mprintf("Adding tone to toned[%d] mess[%d]\n",id,mess[2]);
			toned_add_tone(vr->toned[id], mess);
			break;

		case PC_TONED_ST_DEL:  	
			// call progress detector messages 
			id = mess[2];
			//mprintf("Adding tone to toned[%d] mess[%d]\n",id,mess[2]);
			toned_del_tone(vr->toned[id], mess);
			break;

		case PC_CODEC_GEN:  	
			// general purpose word sent to codec config reg
			id = mess[2];
			//printf("set codec reg ch %d  reg 0x%x val 0x%x\n", id, mess[3], mess[4]);
			pThis->hdaports[id].set_codec_reg(mess[3], mess[4]);
			break;

		case PC_TONED_DEBUG: 
			id = mess[2];
			assert(mess[0] == PC_LTONED_DEBUG);
			toned_start_logging(vr->toned[id], mess);
		break;

		default:
			mprintf("Unknown DSP message [%d]\n", mess[1]);
			//assert(0);
			break;
		}
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: HostDSPSetupPlay
	AUTHOR...: David Rowe
	DATE.....: 11/6/02

	Sets up the codec time slots and OE generator for playing to a port.
	Called just before you want to play.

\*--------------------------------------------------------------------------*/

void HostDSPSetupPlay(int h)
{
	USHORT bd, ch;
	VPBREG *vr;

//	ch = h % V12PCI_PORTS;
//	bd = h / V12PCI_PORTS;
	maphndletodev(h,&bd,&ch);
	vr = vpbreg_get_reg(bd);
	//oe_set(bd, ch+V12PCI_PORTS);
	OpenSwitch	*pThis = (OpenSwitch *)vr->hostdsp;

	pThis->hdaports[ch].set_rx_timeslot(ch+V12PCI_PORTS);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: oe_init
	AUTHOR...: David Rowe
	DATE.....: 17/5/02

	Initialises the Output Enable generator.  This generator controls
        which slots the Host writes to on the TDM bus.

\*--------------------------------------------------------------------------*/

void oe_init(int cardnum) {
	int i;

	// turn off OE generator on slots 0..31
	for(i=0; i<NUM_OE; i++) {
		oe[i] = 0;
		pci_write(pci, cardnum, OE0+i, 0x00);
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: oe_set
	AUTHOR...: David Rowe
	DATE.....: 17/5/02

	Enables a slot on the OE generator.  The PC will now be outputting
	data to this slot.

\*--------------------------------------------------------------------------*/

void oe_set(int bd, int slot) {
	int reg  = slot/BITS_OE;
	int bit  = slot - reg*BITS_OE;
	int mask = 1 << bit;

	oe[reg] |= mask;
	pci_write(pci, bd, OE0+reg, oe[reg]);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: oe_reset
	AUTHOR...: David Rowe
	DATE.....: 17/5/02

	Disables a slot on the OE generator. 

\*--------------------------------------------------------------------------*/

void oe_reset(int bd, int slot) {
	int reg  = slot/BITS_OE;
	int bit  = slot - reg*BITS_OE;
	int mask = 1 << bit;

	oe[reg] ^= mask;
	pci_write(pci, bd, OE0+reg, oe[reg]);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: bitrev
	AUTHOR...: David Rowe
	DATE.....: 5/6/02

	Bit reversal of samples - hardware bug work around.

\*--------------------------------------------------------------------------*/

void bitrev(uchar a[], int n) {
  int i,j;
  uchar b,c;
 
  for(i=0; i<n; i++) {
    b = a[i];
    c = 0;
    for(j=0; j<8; j++) {
      c |= ((b>>j)&0x1) << (7-j);
    }
    a[i] = c;
  }
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: GenerateRing
	AUTHOR...: David Rowe
	DATE.....: 18/8/02

	Generate cadenced ring signal, called periodically to iterate
	state machine. Prevents more than 4 ports ringing at once, to
	prevent overload of power supply.

\*--------------------------------------------------------------------------*/

void GenerateRing(void *d) {
	unsigned long time;
	//int bd, ch;
	int  i, state, next_state;

	OpenSwitch	*pThis = (OpenSwitch *)d;
	VPBREG		*vr=pThis->vr;
	os_chan		* chans= (os_chan*)vr->chans;

	time =  GenerictimeGetTime();	

	for(i=0; i<vr->numch; i++) {
//		ch = i % V12PCI_PORTS;
//		bd = i / V12PCI_PORTS;
	
		state = chans[i].ringgen.state;
		next_state = state;

		switch(state) {
		case RINGGEN_IDLE:
			if ((chans[i].ring == RINGGEN_START) && 
			    (pThis->total_ringing < RINGGEN_MAX_RINGING)) {
				pThis->hdaports[i].set_ring_on();
				chans[i].ringgen.time = time;
				next_state = RINGGEN_RING;
				chans[i].ring = 0;
				pThis->total_ringing++;
			}
			break;
		case RINGGEN_RING:
			if ((time > (chans[i].ringgen.time + RINGGEN_ONTIME))
			    || (chans[i].ring == RINGGEN_STOP)) {
				pThis->hdaports[i].set_ring_off();
				chans[i].ringgen.time = time;
				pThis->total_ringing--;

				// ring gen has been told to stop, or one ring flag set
				if ((chans[i].ring == RINGGEN_STOP) || chans[i].ringgen.one_ring ) {
					next_state = RINGGEN_IDLE;
					chans[i].ring = 0;
				}
				else
					next_state = RINGGEN_PAUSE;
			}
			break;
		case RINGGEN_PAUSE:
			if (chans[i].ring == RINGGEN_STOP) {
				next_state = RINGGEN_IDLE;
				chans[i].ring = 0;
			}
		
			if (time > (chans[i].ringgen.time + RINGGEN_OFFTIME)) {
				pThis->hdaports[i].set_ring_on();
				chans[i].ringgen.time = time;
				next_state = RINGGEN_RING;
				pThis->total_ringing++;
			}
			break;
		}

		chans[i].ringgen.state = next_state;
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: DetectPortConfig
	AUTHOR...: David Rowe
	DATE.....: 22/8/02

	Automatically determines V12PCI configuration of loop/station ports.
	Taking a station port "offhook" will assert the hook detector.

\*--------------------------------------------------------------------------*/

void DetectPortConfig(void *d) {
	int		i;
	int		running=0;
	int		val=0;
//	extern int	vpb_pconf[MAX_BOARDS][BANKB];

	OpenSwitch	*pThis = (OpenSwitch *)d;
	VPBREG		*vr=pThis->vr;

	// Test all ports and set Station or Loop
	for(i=0; i<BANKB; i++) {
		if((val=pThis->hdaports[i].get_codec_reg(0x6))&&(0xa0 == val)){    // codec is present
			pThis->hdaports[i].set_loop_offhook();
			GenericSleep(50);
			running = pci_khook_read(pci, (unsigned short)vr->cardtypnum, i);
			pThis->hdaports[i].set_loop_onhook();
			if (PORT_DET_THRESH < running) {
				pThis->hdaports[i].set_config(HDAPORT_STATION);
				vpb_pconf[vr->cardnum][i]=HDAPORT_STATION;
				mprintf("bd[%d] Port[%d] configured as STATION (running=%d)\n",vr->cardnum,i,running);
			}
			else {
				pThis->hdaports[i].set_config(HDAPORT_LOOP);
				vpb_pconf[vr->cardnum][i]=HDAPORT_LOOP;
				mprintf("bd[%d] Port[%d] configured as LOOP (running=%d)\n", vr->cardnum,i,running);	
			}
		}
		else {
			pThis->hdaports[i].set_config(HDAPORT_ABSENT);
			vpb_pconf[vr->cardnum][i]=HDAPORT_ABSENT;
			mprintf("bd[%d] Port[%d] Codec absent\n", vr->cardnum,i);
		}
	}
}


/*--------------------------------------------------------------------------*\

	FUNCTION.: GenerateFlash
	AUTHOR...: David Rowe
	DATE.....: 27/8/02

	Generate hook flash while dialing.

\*--------------------------------------------------------------------------*/

void GenerateFlash(void *d) {
	unsigned long time, start_time, run_time, duration;
	//int           bd, ch;
	int           i, state;
	word          mess[DSP_LCODEC_BKEN];
	OpenSwitch	*pThis = (OpenSwitch *)d;
	VPBREG		*vr=pThis->vr;
	os_chan		* chans = (os_chan *)vr->chans;

	time =  GenerictimeGetTime();	

	for(i=0; i<vr->numch; i++) {
//		ch = i % V12PCI_PORTS;
//		bd = i / V12PCI_PORTS;
	
		state = chans[i].flash.state;
		start_time = chans[i].flash.start_time;
		duration = chans[i].flash.duration;
		
		if (state == FLASH_ACTIVE) {
			run_time = time - start_time;
			mprintf("run_time = %ld duration = %ld", run_time, duration);
			if (run_time > duration) {
				pThis->hdaports[i].set_loop_offhook();
				state = FLASH_INACTIVE;
				mess[0] = DSP_LCODEC_BKEN;
				mess[1] = DSP_CODEC_BKEN;
				mess[2] = i;
				vr->upmess->Write(mess, mess[0]);			       
			}
		}

		chans[i].flash.state = state;
	}
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: toned_callback
	AUTHOR...: David Rowe
	DATE.....: 14/1/03

	This function is called by the tone detector when a tone is detected
	or a tone detector log message is generated.

\*--------------------------------------------------------------------------*/

void toned_callback(word mess[], void * board) {

	// convert from absolute channel number to bd, ch
	VPBREG 	*vr = (VPBREG *)board;

//	mprintf("GOT TONE FROM[%d]\n",mess[2]);
//	int ch = mess[2] % V12PCI_PORTS;
//	int bd = mess[2] / V12PCI_PORTS;

//	mess[2] = ch;
	vr->upmess->Write(mess, mess[0]);		
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: read_iic()
	AUTHOR...: Peter Wintulich
	DATE.....: 18/8/04

        Read I2C port.
\*--------------------------------------------------------------------------*/
int Read_Iic(unsigned short card, unsigned short addr, unsigned short length, unsigned short *buf)
{
	int ret;

	ret = pci_block_iicread(pci, card, addr, length, buf);
	//mprintf("%d = read_iic(0x%04x, ...)\n",ret,addr);
	return (ret);
}

/*--------------------------------------------------------------------------*\

	FUNCTION.: write_iic()
	AUTHOR...: Peter Wintulich
	DATE.....: 18/8/04

	Write I2C port.

\*--------------------------------------------------------------------------*/
int Write_Iic( unsigned short card, unsigned short addr, unsigned short length, unsigned short *buf)
{
	int ret;

	ret = pci_block_iicwrite(pci, card, addr, length, buf);
	//mprintf("%d = write_iic(0x%04x, ...)\n",ret,addr);
	return (ret);
}



