/*---------------------------------------------------------------------------*\

	FILE....: utilfuncs.cpp
	AUTHOR..: Ben Kramer
	DATE....: 26/07/2004
	DESC....: Contains a bunch of common utility functions

\*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\

         Voicetronix Voice Processing Board (VPB) Software

         Copyright (C) 1999-2004, 2006 Voicetronix www.voicetronix.com.au

         This library is free software; you can redistribute it and/or
         modify it under the terms of the GNU Lesser General Public
         License as published by the Free Software Foundation; either
         version 2.1 of the License, or (at your option) any later version.

         This library is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
         Lesser General Public License for more details.

         You should have received a copy of the GNU Lesser General Public
         License along with this library; if not, write to the Free Software
         Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	 USA

\*---------------------------------------------------------------------------*/

#include "utilfuncs.h"
#include "llconf.h"
#include "mess.h"
#include "vpbapi.h"
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>



/* Conf file reading states */
#define C_START		0
#define C_SECT		1
#define C_DATA 		2
#define C_SUBSECT	3
/* section types */
#define CS_NONE 	0
#define CS_GEN		1
#define CS_CARD 	2
#define CS_CHAN		3


/* internal function defns */
static void trim_buffer(char * buffer);
static int conf_get_section(char * buffer);
static int conf_save_buf(int file_type, CONF_INFO * conf_pointer, int state, int sect, int subsect, int cardno, char *buffer);
static int conf_get_tag_value(char * tag,char * value,char * buffer);
static int conf_get_cardno(char * buffer);
//static void conf_dumper(CONF_INFO * conf_p);
static void conf_save_tone(CONF_INFO * conf_p, char * tone);
static void conf_dump_print(CONF_INFO * conf_p);
//static void rolllogger(const char* const dir, const char* const log_name);

/* gobal variables */
//static FILE *logfileh;
//static int logging;
//static char logstamp[9];
//static char logfilepath[256];	// path to log file
//static char logfilename[254];

llc_list *vpb_read_confll(CONF_INFO * cp)
{
	int cardcount=0;
	int i;
	char file[]="/etc/vpb/vpb.conf";
	char *tmp;
	int tmpi;
	char name[64];
	char confval[64];
	float hw_play_gain;
	float hw_rec_gain;
	float play_gain;
	float rec_gain;
	unsigned int bal1;
	unsigned int bal2;
	unsigned int bal3;
	int anatype;
	int channel;
	int settings;
	int cardtype;
	int channels;
	int encoding=-1;
	int bdno;	// board number
	int bdty;	// board type
	int bdtyno;	// board type number

	llc_list *list      = NULL;
	llc_cat  *cat       = NULL;
	llc_var  *var       = NULL;
	llc_list *list_root = conf_load(file);

	if( ! list_root ) return NULL;

	mprintf("vpb_read_confll: loaded [%s]\n",file);
	/* Lets fill in the general stuff */
	list = conf_find_list(list_root,"general");
	if (list != NULL){
		cat = conf_find_cat(list->cats,"general");
		cp->verbosity = atoi( conf_find_val(confval, cat,"verbosity") );
		mprintf("vpb_read_confll: verbosity = [%s]%d\n",confval,cp->verbosity);
		conf_find_val(cp->logdir, cat, "logdir");
		cp->country_code = atoi( conf_find_val(confval, cat, "country") );
		cp->cardcount = atoi( conf_find_val(confval, cat, "cardcount") );
		mprintf("vpb_read_confll: cardcount = [%s]%d\n",confval,cp->cardcount);
		cat = conf_find_cat(list->cats,"cards");
		var = cat->vars;
		while (var != NULL){
			bdno = atoi(var->name);	// board number
			bdty=-1;		// board type
			bdtyno=-1;		// board type number
			if (strncasecmp(var->value,"openpri",7)==0){
				bdty=VPB_PRI;
			}
			else if (strncasecmp(var->value,"openswi",7)==0){
				bdty=VPB_V12PCI;
			}
			else if (strncasecmp(var->value,"openlin",7)==0){
				bdty=VPB_V4PCI;
			}
			else if (strncasecmp(var->value,"openlog",7)==0){
				bdty=VPB_V4LOG;
			}
			else if (strncasecmp(var->value,"openpci",7)==0){
				bdty=VPB_OPCI;
			}
			if (bdty != -1){
				tmp=strstr(var->value,":");
				bdtyno=atoi(tmp+1);
				if (((bdno >-1)&&(bdno <254)) && ((bdtyno>-1)&&(bdtyno<30))){
					cp->cards[bdno].type=bdty;
					cp->cards[bdno].number=bdtyno;
				}
				else {
					mprintf("Invalid board number or board type number![%d][%d]\n",bdty,bdtyno);
				}
			}
			else {
				mprintf("Couldnt work out board type!\n");
			}
			var = var->next;
		}
		cp->tone_count=0;
	}
	else {
		mprintf("vpb_read_confll: Couldnt find the \"general\" section!\n");
		conf_release_list( list_root );
		return NULL;
	}

	/* Lets check out the OpenLine stuff */
	list = conf_find_list(list_root,"openline");
	if (list != NULL){
		mprintf("vpb_read_confll: Scanning OpenLine conf..\n");
		cat = conf_find_cat(list->cats,"general");
		cp->OpenLine.cards = atoi( conf_find_val(confval, cat, "cards") );
		if (cp->OpenLine.cards >0){
			cardcount = cardcount + cp->OpenLine.cards;
			cp->OpenLine.channels = atoi( conf_find_val(confval, cat,"channels") );
			/* OK, check out the card specs */
			settings=0;
			for (i=0;i<cp->OpenLine.cards;i++){
				hw_play_gain=0;
				hw_rec_gain=0;
				play_gain=0;
				rec_gain=0;
				bal1=0;
				bal2=0;
				bal3=0;
				settings=0;
				channels=4;
				cp->OpenLine.card[i].channels=channels;
				sprintf(name,"card%d",i);
				cat = conf_find_cat(list->cats,name);
				var = cat->vars;
				cardtype = VPB_V4PCI;
				cp->OpenLine.card[i].type=cardtype;
				cp->OpenLine.card[i].cardnum=i;
				while (var != NULL){
					if (strcasecmp(var->name,"bal1")==0){
						//bal1 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal1);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"bal2")==0){
						//bal2 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal2);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"bal3")==0){
						//bal3 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal3);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"hwplaygain")==0){
						hw_play_gain = atof(var->value);
						settings |= MOD_HWPG;
					}
					else if (strcasecmp(var->name,"hwrecgain")==0){
						hw_rec_gain = atof(var->value);
						settings |= MOD_HWRG;
					}
					else if (strcasecmp(var->name,"playgain")==0){
						play_gain = atof(var->value);
						settings |= MOD_PG;
					}
					else if (strcasecmp(var->name,"recgain")==0){
						rec_gain = atof(var->value);
						settings |= MOD_RG;
					}
					else if (strcasecmp(var->name,"channels")==0){
						channels = atoi(var->value);
						cp->OpenLine.card[i].channels=channels;
					}
					else if (strcasecmp(var->name,"type")==0){
						if (0 == strncasecmp(var->value,"openli",6)){
							cardtype = VPB_V4PCI;
						}
						else if (0 == strncasecmp(var->value,"openlo",6)){
							cardtype = VPB_V4LOG;
						}
						cp->OpenLine.card[i].type=cardtype;
					}
					else if (strncasecmp(var->name,"channel",7)==0){
						if (strcasecmp(var->value,"fxo")==0){
							anatype = VPB_FXO;
						}
						else {
							anatype = VPB_FXS;
						}
						tmp=strtok(var->name,"channel");
						channel=atoi(tmp);
						/* Save channel information */
						cp->OpenLine.card[i].iface[channel].channel = channel;
						cp->OpenLine.card[i].iface[channel].type = anatype;
						cp->OpenLine.card[i].iface[channel].hw_play_gain = hw_play_gain;
						cp->OpenLine.card[i].iface[channel].hw_rec_gain = hw_rec_gain;
						cp->OpenLine.card[i].iface[channel].play_gain = play_gain;
						cp->OpenLine.card[i].iface[channel].rec_gain = rec_gain;
						cp->OpenLine.card[i].iface[channel].codec_hyb_bal1 = (unsigned short)bal1;
						cp->OpenLine.card[i].iface[channel].codec_hyb_bal2 = (unsigned short)bal2;
						cp->OpenLine.card[i].iface[channel].codec_hyb_bal3 = (unsigned short)bal3;
						cp->OpenLine.card[i].iface[channel].settings = settings;
					}
					var = var->next;
				}
			}
		}
	}
	else {
		cp->OpenLine.cards=0;
		cp->OpenLine.channels=0;
	}
	/* Lets check out the OpenSwitch stuff */
	list = conf_find_list(list_root,"openswitch");
	if (list != NULL){
		mprintf("vpb_read_confll: Scanning OpenSwitch conf..\n");
		cat = conf_find_cat(list->cats,"general");
		cp->OpenSwitch.cards = atoi( conf_find_val(confval, cat, "cards") );
		if (cp->OpenSwitch.cards >0){
			cardcount = cardcount + cp->OpenSwitch.cards;
			cp->OpenSwitch.channels = atoi( conf_find_val(confval, cat, "channels") );
			for (i=0;i<cp->OpenSwitch.cards;i++){
				hw_play_gain=0;
				hw_rec_gain=0;
				play_gain=0;
				rec_gain=0;
				bal1=0;
				bal2=0;
				bal3=0;
				settings=0;
				channels=12;
				anatype=-1;
				channel=-1;
				cardtype=VPB_V12PCI;
				sprintf(name,"card%d",i);
				mprintf("Looking for cat for [%s]\n",name);
				cat = conf_find_cat(list->cats,name);
				var = cat->vars;
				cp->OpenSwitch.card[i].cardnum=i;
				while (var != NULL){
					if (strcasecmp(var->name,"bal1")==0){
						//bal1 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal1);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"bal2")==0){
						//bal2 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal2);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"bal3")==0){
						//bal3 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal3);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"hwplaygain")==0){
						hw_play_gain = atof(var->value);
						settings |= MOD_HWPG;
					}
					else if (strcasecmp(var->name,"hwrecgain")==0){
						hw_rec_gain = atof(var->value);
						settings |= MOD_HWRG;
					}
					else if (strcasecmp(var->name,"playgain")==0){
						play_gain = atof(var->value);
						settings |= MOD_PG;
					}
					else if (strcasecmp(var->name,"recgain")==0){
						rec_gain = atof(var->value);
						settings |= MOD_RG;
					}
					else if (strcasecmp(var->name,"channels")==0){
						channels = atoi(var->value);
						cp->OpenSwitch.card[i].channels=channels;
					}
					else if (strcasecmp(var->name,"type")==0){
						if (0 == strncasecmp(var->value,"openswitch1",11)){
							cardtype = VPB_V12PCI;
						}
						else if (0 == strncasecmp(var->value,"openswitch6",11)){
							cardtype = VPB_V6PCI;
						}
						cp->OpenSwitch.card[i].type=cardtype;
					}
					else if (strncasecmp(var->name,"channel",7)==0){
						if (strcasecmp(var->value,"fxo")==0){
							anatype = VPB_FXO;
						}
						else {
							anatype = VPB_FXS;
						}
						tmp=strtok(var->name,"channel");
						channel=atoi(tmp);
						/* Save channel information */
						cp->OpenSwitch.card[i].iface[channel].channel = channel;
						cp->OpenSwitch.card[i].iface[channel].type = anatype;
						cp->OpenSwitch.card[i].iface[channel].hw_play_gain = hw_play_gain;
						cp->OpenSwitch.card[i].iface[channel].hw_rec_gain = hw_rec_gain;
						cp->OpenSwitch.card[i].iface[channel].play_gain = play_gain;
						cp->OpenSwitch.card[i].iface[channel].rec_gain = rec_gain;
						cp->OpenSwitch.card[i].iface[channel].codec_hyb_bal1 = (unsigned short)bal1;
						cp->OpenSwitch.card[i].iface[channel].codec_hyb_bal2 = (unsigned short)bal2;
						cp->OpenSwitch.card[i].iface[channel].codec_hyb_bal3 = (unsigned short)bal3;
						cp->OpenSwitch.card[i].iface[channel].settings = settings;
						channel=-1;
						anatype=-1;
					}
					var = var->next;
				}
			}
		}
	}
	else {
		cp->OpenSwitch.cards=0;
		cp->OpenSwitch.channels=0;
	}
	hw_play_gain=0;
	hw_rec_gain=0;
	play_gain=0;
	rec_gain=0;
	/* Lets check out the OpenPri stuff */
	list = conf_find_list(list_root,"openpri");
	if (list != NULL){
		mprintf("vpb_read_confll: Scanning OpenPri conf..\n");
		cat = conf_find_cat(list->cats,"general");
		cp->OpenPri.cards = atoi( conf_find_val(confval, cat, "cards") );
		if (cp->OpenPri.cards >0){
			cardcount = cardcount + cp->OpenPri.cards;
			cp->OpenPri.channels = atoi( conf_find_val(confval, cat, "channels") );
			for (i=0;i<cp->OpenPri.cards;i++){
				settings=0;
				cp->OpenPri.card[i].node=ISDN_NODE_CLIENT;
				sprintf(name,"card%d",i);
				cat = conf_find_cat(list->cats,name);
				var = cat->vars;
				while (var != NULL){
					if (strcasecmp(var->name,"hwplaygain")==0){
						hw_play_gain = atof(var->value);
						settings |= MOD_HWPG;
					}
					else if (strcasecmp(var->name,"hwrecgain")==0){
						hw_rec_gain = atof(var->value);
						settings |= MOD_HWRG;
					}
					else if (strcasecmp(var->name,"playgain")==0){
						play_gain = atof(var->value);
						settings |= MOD_PG;
					}
					else if (strcasecmp(var->name,"recgain")==0){
						rec_gain = atof(var->value);
						settings |= MOD_RG;
					}
					else if(strcasecmp(var->name,"name")==0){
						strcpy((char *)cp->OpenPri.card[i].name,var->value);
					}
					else if(strcasecmp(var->name,"iftype")==0){
						if (strcasecmp(var->value,"e1")==0){
							cp->OpenPri.card[i].if_type=VPB_PRI_E1;
						}
						else {
							cp->OpenPri.card[i].if_type=VPB_PRI_T1;
						}
					}
					else if(strcasecmp(var->name,"dchannel")==0){
						cp->OpenPri.card[i].d_channel=atoi(var->value);
					}
					else if(strcasecmp(var->name,"priverbose")==0){
						cp->OpenPri.card[i].priverbose=atoi(var->value);
					}
					else if(strcasecmp(var->name,"encoding")==0){
						if(strcasecmp(var->value,"ALAW")==0){
							encoding=VPB_ALAW;
						}
						else if(strcasecmp(var->value,"ULAW")==0){
							encoding=VPB_MULAW;
						}
						else if(strcasecmp(var->value,"MULAW")==0){
							encoding=VPB_MULAW;
						}
						else if(strcasecmp(var->value,"raw")==0){
							encoding=VPB_RAW;
						}
						//cp->OpenPri.card[i].encoding=tmpi;
					}
					else if(strcasecmp(var->name,"dialplan")==0){
						tmpi=ISDN_DP_UNKNOWN;
						if(strcasecmp(var->value,"national")==0){
							tmpi=ISDN_DP_NATIONAL_ISDN;
						}
						else if(strcasecmp(var->value,"international")==0){
							tmpi=ISDN_DP_INTERNATIONAL_ISDN;
						}
						else if(strcasecmp(var->value,"local")==0){
							tmpi=ISDN_DP_LOCAL_ISDN;
						}
						else if(strcasecmp(var->value,"private")==0){
							tmpi=ISDN_DP_PRIVATE;
						}
						cp->OpenPri.card[i].dialplan=tmpi;
					}
					else if(strcasecmp(var->name,"localdialplan")==0){
						tmpi=ISDN_DP_UNKNOWN;
						if(strcasecmp(var->value,"national")==0){
							tmpi=ISDN_DP_NATIONAL_ISDN;
						}
						else if(strcasecmp(var->value,"international")==0){
							tmpi=ISDN_DP_INTERNATIONAL_ISDN;
						}
						else if(strcasecmp(var->value,"local")==0){
							tmpi=ISDN_DP_LOCAL_ISDN;
						}
						else if(strcasecmp(var->value,"private")==0){
							tmpi=ISDN_DP_PRIVATE;
						}
						cp->OpenPri.card[i].localdialplan=tmpi;
					}
					else if(strcasecmp(var->name,"protocol")==0){
						if(strcasecmp(var->value,"NI2")==0){
							tmpi=ISDN_PROTO_NI2;
						}
						else if(strcasecmp(var->value,"DMS100")==0){
							tmpi=ISDN_PROTO_DMS100;
						}
						else if(strcasecmp(var->value,"LUCENT5E")==0){
							tmpi=ISDN_PROTO_LUCENT5E;
						}
						else if(strcasecmp(var->value,"ATT4ESS")==0){
							tmpi=ISDN_PROTO_ATT4ESS;
						}
						else if(strcasecmp(var->value,"EUROISDN_E1")==0){
							tmpi=ISDN_PROTO_EUROISDN_E1;
						}
						else if(strcasecmp(var->value,"EUROISDN_T1")==0){
							tmpi=ISDN_PROTO_EUROISDN_T1;
						}
						else if(strcasecmp(var->value,"NI1")==0){
							tmpi=ISDN_PROTO_NI1;
						}
						else if(strcasecmp(var->value,"GR303_EOC")==0){
							tmpi=ISDN_PROTO_GR303_EOC;
						}
						else if(strcasecmp(var->value,"GR303_TMC")==0){
							tmpi=ISDN_PROTO_GR303_TMC;
						}
						else
							tmpi=ISDN_PROTO_UNKNOWN;

						cp->OpenPri.card[i].protocol=tmpi;
					}
					else if(strcasecmp(var->name,"node")==0){
						if(strcasecmp(var->value,"network")==0){
							tmpi=ISDN_NODE_NETWORK;
						} else if(strcasecmp(var->value,"log")==0){
							tmpi=ISDN_NODE_LOG;
						} else {
							tmpi=ISDN_NODE_CLIENT;
						}
						cp->OpenPri.card[i].node=tmpi;
					}
					else if(strncasecmp(var->name,"channel",7)==0){
						tmp=strtok(var->name,"channel");
						tmpi=atoi(tmp);
						if( tmpi > 0 && tmpi < cp->OpenPri.channels + 2 ){
							cp->OpenPri.card[i].iface[tmpi].channel=tmpi;
							strcpy((char *)cp->OpenPri.card[i].iface[tmpi].name,
							       var->value);
							strcpy((char *)cp->OpenPri.card[i].iface[tmpi].card,
							       (char *)cp->OpenPri.card[i].name);
							cp->OpenPri.card[i].iface[tmpi].encoding =
								(encoding == -1) ? VPB_ALAW : encoding;
						} else
							mprintf("Invalid channel number %d\n", tmpi);
					}
				var = var->next;
				}
				cp->OpenPri.card[i].hw_play_gain = hw_play_gain;
				cp->OpenPri.card[i].hw_rec_gain = hw_rec_gain;
				cp->OpenPri.card[i].play_gain = play_gain;
				cp->OpenPri.card[i].rec_gain = rec_gain;
				cp->OpenPri.card[i].settings = settings;
			}
		}
	}
	else {
		cp->OpenPri.cards=0;
		cp->OpenPri.channels=0;
	}
	/* Lets check out the OpenPci stuff */
	/* Moved to the vtcore structure
	list = conf_find_list(list_root,"openpci");
	if (list != NULL){
		mprintf("vpb_read_confll: Scanning OpenPci conf..\n");
		cat = conf_find_cat(list->cats,"general");
		cp->OpenPci.cards = atoi( conf_find_val(confval, cat, "cards") );
		if (cp->OpenPci.cards >0){
			cardcount = cardcount + cp->OpenPci.cards;
			cp->OpenPci.channels = atoi( conf_find_val(confval, cat, "channels") );
			for (i=0;i<cp->OpenPci.cards;i++){
				hw_play_gain=0;
				hw_rec_gain=0;
				play_gain=0;
				rec_gain=0;
				bal1=0;
				bal2=0;
				bal3=0;
				settings=0;
				channels=12;
				anatype=-1;
				channel=-1;
				cardtype=VPB_OPCI;
				sprintf(name,"card%d",i);
				mprintf("Looking for cat for [%s]\n",name);
				cat = conf_find_cat(list->cats,name);
				if(cat)
					var = cat->vars;
				cp->OpenPci.card[i].cardnum=i;
				while (var != NULL){
					if (strcasecmp(var->name,"bal1")==0){
						//bal1 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal1);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"bal2")==0){
						//bal2 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal2);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"bal3")==0){
						//bal3 = (unsigned short)atoi(var->value);
						sscanf(var->value,"%x",&bal3);
						settings |= MOD_BALS;
					}
					else if (strcasecmp(var->name,"hwplaygain")==0){
						hw_play_gain = atof(var->value);
						settings |= MOD_HWPG;
					}
					else if (strcasecmp(var->name,"hwrecgain")==0){
						hw_rec_gain = atof(var->value);
						settings |= MOD_HWRG;
					}
					else if (strcasecmp(var->name,"playgain")==0){
						play_gain = atof(var->value);
						settings |= MOD_PG;
					}
					else if (strcasecmp(var->name,"recgain")==0){
						rec_gain = atof(var->value);
						settings |= MOD_RG;
					}
					else if (strcasecmp(var->name,"channels")==0){
						channels = atoi(var->value);
						cp->OpenPci.card[i].channels=channels;
					}
					else if (strcasecmp(var->name,"type")==0){
						if (0 == strncasecmp(var->value,"openpci",7)){
							cardtype = VPB_OPCI;
						}
						cp->OpenPci.card[i].type=cardtype;
					}
					var = var->next;
				}
			}
		}
	}
	else {
		cp->OpenPci.cards=0;
		cp->OpenPci.channels=0;
	}
	*/
	/* Lets check out the vtcore stuff */
	list = conf_find_list(list_root,"vtcore");
	if (list != NULL){
		mprintf("vpb_read_confll: Scanning vtcore conf..\n");
		cat = conf_find_cat(list->cats,"general");
		cp->vtcore.cards = atoi( conf_find_val(confval, cat, "cards") );
		if (cp->vtcore.cards >0){
			cp->vtcore.card = new VTCORE_CARD[cp->vtcore.cards];
			cardcount = cardcount + cp->vtcore.cards;
			cp->vtcore.channels = atoi( conf_find_val(confval, cat,"channels") );
			for (i=0;i<cp->vtcore.cards;i++){
				sprintf(name,"card%d",i);
				mprintf("Looking for cat for [%s]\n",name);
				cat = conf_find_cat(list->cats,name);
				if(cat)
					var = cat->vars;
				cp->vtcore.card[i].cardnum=i;
				cp->vtcore.card[i].config=(void *)var;
				while (var != NULL){
					if (strcasecmp(var->name,"channels")==0){
						channels = atoi(var->value);
						cp->vtcore.card[i].channels=channels;
					}
					else if (strcasecmp(var->name,"type")==0){
						if (0 == strncasecmp(var->value,"openpci",7)){
							cp->vtcore.card[i].type=VPB_OPCI;
						}
						else if (0 == strncasecmp(var->value,"openswitch",7)){
							cp->vtcore.card[i].type=VPB_OSW;
						}
					}
					var = var->next;
				}
			}
		}
	}
	else {
		cp->vtcore.cards=0;
		cp->vtcore.channels=0;
	}
//	conf_dumper(cp);
	conf_dump_print(cp);

	return list_root;
}

/******************************************************************************
	FUNC: vpb_read_conf
	DESC: Reads a conf file in to the conf_info structure pointer passed
	AUTH: Ben Kramer
	DATE: 26/07/2004
******************************************************************************/
int vpb_read_conf(CONF_INFO * conf_pointer, char * file_name, int file_type)
{
	FILE *conf_file;
	char buffer[128];
	char * buf_p;
	int state=C_START;
	int sect=0;
	int newsect=0;
	int subsect=0;
	int count=0;
	int ret=0;
	int cardno=0;

	/* Initialize some things */
	/*
	conf_pointer->tone_count=0;
	conf_pointer->logdir[0]='\0';
	conf_pointer->OpenPri.cards=0;
	conf_pointer->OpenSwitch.cards=0;
	conf_pointer->OpenLine.cards=0;
	*/

	buf_p=buffer;

	conf_file = fopen(file_name,"r");
	if (conf_file == NULL){
		return -1;
	}
	else {
		while ((count < 128)&&(fread(buf_p,sizeof(char),1,conf_file)!=0)){
			count++;
			//mprintf("%c",*buf_p);
			if (*buf_p == 10){
				*buf_p='\0';
				trim_buffer(buffer);
//				mprintf("[%s]\n",buffer);
				if ((buffer[0] != '#')&&(buffer[0] != '\0')){
					/* we have a non-comment line */
					if (buffer[0] == '['){
						newsect=conf_get_section(buffer);
//						mprintf("SECTION[%d]\n",newsect);
						if ((state !=C_SECT)&&(state != C_SUBSECT)){
							state=C_SECT;
							sect=newsect;
						}
						else if(state == C_SECT) {
							if ((sect == CS_GEN)||(sect = CS_CARD)){
								sect=newsect;
							}
							else {
								state=C_SUBSECT;
								sect=newsect;
							}
						}
						/* Check if we are in a card section and grab its number */
						if (sect == CS_CARD){
							cardno=conf_get_cardno(buffer);
						}
					}
					else {
						if ((state !=C_SECT)&&(state != C_SUBSECT)){
							/* DANGER we are not in a section to store the info! */
//							mprintf("Cant save info, not in section!\n");
						}
						else {
							ret = conf_save_buf(file_type,conf_pointer,state,sect,subsect,cardno,buffer);
							if (ret !=0){
//								mprintf("PROBLEM: Couldnt save info!!\n");
							}
						}
					}
				}
				count=0;
				buf_p=buffer;
			}
			else {
				buf_p++;
			}
		}
	}
	fclose(conf_file);
//	conf_dumper(conf_pointer);
	conf_dump_print(conf_pointer);
	return 0;
}

void trim_buffer(char * buffer)
{
	char * buf_p=buffer;
	int start;
	int finish;
	int state=0;
	int count=0;
	int i;

	if (*buf_p == '\0'){
		return;
	}
//	/* Find start */
	while(*buf_p != '\0')
	{
		if ( ((*buf_p > 32) && (*buf_p<127)) && (state == 0))
		{
			start=count;
			state=1;
//			mprintf("== found start of text at [%d][%c][%d]\n",count,*buf_p,short(*buf_p));
		}
		buf_p++;
		count++;
	}
//	/* find finish */
	state=0;
	while (count>0){
		buf_p--;
		count--;
		if ( ((*buf_p > 32)&&(*buf_p<127)) && (state == 0)){
			finish=count;
//			mprintf("== found finish of text at [%d][%c][%d]\n",count,*buf_p,short(*buf_p));
			state=1;
		}
	}
	/* move from start to finish inside buffer */
	buf_p=buffer;
	for(i=start;i<=finish;i++){
		*buf_p=buffer[i];
		buf_p++;
	}
	*buf_p='\0';
}

int conf_get_section(char * buffer)
{
	char sectbuf[126];
	char * buf_p;
	char * sect_p;
	int count;
	buf_p=buffer;
	sect_p=sectbuf;
	buf_p++;
	count=0;
	while((count<126)&&(*buf_p != ']')){
		count++;
		*sect_p=*buf_p;
		sect_p++;
		buf_p++;
	}
	*(sect_p++)='\0';
	if(strncasecmp(sectbuf,"general",7)==0){
		return CS_GEN;
	}
	if(strncasecmp(sectbuf,"card",4)==0){
		return CS_CARD;
	}
	else {
		return CS_NONE;
	}
}

static int conf_get_cardno(char * buffer)
{
	char sectbuf[126];
	char * buf_p;
	char * sect_p;
	int count;
	int tmp;
	char * tmp_p;
	buf_p=buffer;
	sect_p=sectbuf;
	buf_p++;
	count=0;
	while((count<126)&&(*buf_p != ']')){
		count++;
		*sect_p=*buf_p;
		sect_p++;
		buf_p++;
	}
	*(sect_p++)='\0';
	if(strncasecmp(sectbuf,"card",4)==0){
		tmp_p=strtok(sectbuf,"card");
		tmp=atoi(tmp_p);
		return tmp;
	}
	else {
		return -1;
	}
}

int conf_save_buf(int file_type, CONF_INFO * conf_p, int state, int sect, int subsect, int cardno, char *buffer)
{
	char tag[64],value[64];
	int ret;
	int tmp;
	char * tmp_p;
	ret=conf_get_tag_value(tag,value,buffer);
//	mprintf("sect[%d] tag[%s] val[%s]\n",sect,tag,value);
	switch(state){
		case C_SECT:
			if ((sect == CS_GEN)&&(file_type == VPB_CONF_GEN)){
				if(strcasecmp(tag,"logdir")==0){
					strcpy(conf_p->logdir,value);
					return 0;
				}
				if(strcasecmp(tag,"country")==0){
					conf_p->country_code=atoi(value);
					return 0;
				}
				if(strcasecmp(tag,"verbosity")==0){
					conf_p->verbosity=atoi(value);
					return 0;
				}
				if(strcasecmp(tag,"tone")==0){
					conf_save_tone(conf_p,value);
					return 0;
				}
				if(strcasecmp(tag,"cardcount")==0){
					conf_p->cardcount=atoi(value);
					return 0;
				}
			}
			else if ((sect == CS_GEN)&&(file_type == VPB_CONF_OP)){
				if(strcasecmp(tag,"channels")==0){
					conf_p->OpenPri.channels=atoi(value);
					return 0;
				}
				if(strcasecmp(tag,"cards")==0){
					conf_p->OpenPri.cards=atoi(value);
					return 0;
				}
			}
			else if ((sect == CS_GEN)&&(file_type == VPB_CONF_OL)){
				if(strcasecmp(tag,"channels")==0){
					conf_p->OpenLine.channels=atoi(value);
					return 0;
				}
				if(strcasecmp(tag,"cards")==0){
					conf_p->OpenLine.cards=atoi(value);
					return 0;
				}
			}
			else if ((sect == CS_GEN)&&(file_type == VPB_CONF_OS)){
				if(strcasecmp(tag,"channels")==0){
					conf_p->OpenSwitch.channels=atoi(value);
					return 0;
				}
				if(strcasecmp(tag,"cards")==0){
					conf_p->OpenSwitch.cards=atoi(value);
					return 0;
				}
			}
			else if ((sect == CS_CARD)&&(file_type == VPB_CONF_GEN)){
				int bdno = atoi(tag);	// board number
				int bdty=-1;		// board type
				int bdtyno=-1;		// board type number
				if (strncasecmp(value,"openpri",7)==0){
					bdty=VPB_CONF_OP;
				}
				else if (strncasecmp(value,"openswi",7)==0){
					bdty=VPB_CONF_OS;
				}
				else if (strncasecmp(value,"openlin",7)==0){
					bdty=VPB_CONF_OL;
				}
				if (bdty != -1){
					tmp_p=strstr(value,":");
					bdtyno=atoi(tmp_p+1);
					if (((bdno >-1)&&(bdno <254)) && ((bdtyno>0)&&(bdtyno<30))){
						conf_p->cards[bdno].type=bdty;
						conf_p->cards[bdno].number=bdtyno;
						return 0;
					}
					else {
						mprintf("Invalid board number or board type number!\n");
					}
				}
				else {
					mprintf("Couldnt work out board type!\n");
				}
				return -1;
			}
			else if ((sect == CS_CARD)&&(file_type == VPB_CONF_OP)){
				if(strcasecmp(tag,"name")==0){
					strcpy((char *)conf_p->OpenPri.card[cardno].name,value);
					return 0;
				}
				else if(strcasecmp(tag,"iftype")==0){
					if (strcasecmp(value,"e1")==0){
						conf_p->OpenPri.card[cardno].if_type=VPB_PRI_E1;
					}
					else {
						conf_p->OpenPri.card[cardno].if_type=VPB_PRI_T1;
					}
					return 0;
				}
				else if(strcasecmp(tag,"dchannel")==0){
					conf_p->OpenPri.card[cardno].d_channel=atoi(value);
					return 0;
				}
				else if(strcasecmp(tag,"priverbose")==0){
					conf_p->OpenPri.card[cardno].priverbose=atoi(value);
					return 0;
				}
				else if(strcasecmp(tag,"encoding")==0){
					tmp=VPB_ALAW;
					if(strcasecmp(value,"ALAW")==0){
						tmp=VPB_ALAW;
					}
					else if(strcasecmp(value,"ULAW")==0){
						tmp=VPB_MULAW;
					}
					else if(strcasecmp(value,"MULAW")==0){
						tmp=VPB_MULAW;
					}
					//conf_p->OpenPri.card[cardno].encoding=tmp;
					return 0;
				}
				else if(strcasecmp(tag,"protocol")==0){
					if(strcasecmp(value,"NI2")==0){
						tmp=ISDN_PROTO_NI2;
					}
					else if(strcasecmp(value,"DMS100")==0){
						tmp=ISDN_PROTO_DMS100;
					}
					else if(strcasecmp(value,"LUCENT5E")==0){
						tmp=ISDN_PROTO_LUCENT5E;
					}
					else if(strcasecmp(value,"ATT4ESS")==0){
						tmp=ISDN_PROTO_ATT4ESS;
					}
					else if(strcasecmp(value,"EUROISDN_E1")==0){
						tmp=ISDN_PROTO_EUROISDN_E1;
					}
					else if(strcasecmp(value,"EUROISDN_T1")==0){
						tmp=ISDN_PROTO_EUROISDN_T1;
					}
					else if(strcasecmp(value,"NI1")==0){
						tmp=ISDN_PROTO_NI1;
					}
					else if(strcasecmp(value,"GR303_EOC")==0){
						tmp=ISDN_PROTO_GR303_EOC;
					}
					else if(strcasecmp(value,"GR303_TMC")==0){
						tmp=ISDN_PROTO_GR303_TMC;
					}
					else
						tmp=ISDN_PROTO_UNKNOWN;

					conf_p->OpenPri.card[cardno].protocol=tmp;
					return 0;
				}
				else if(strcasecmp(tag,"node")==0){
					if(strcasecmp(value,"network")==0){
						tmp=ISDN_NODE_NETWORK;
					} else if(strcasecmp(value,"log")==0){
						tmp=ISDN_NODE_LOG;
					} else {
						tmp=ISDN_NODE_CLIENT;
					}
					conf_p->OpenPri.card[cardno].node=tmp;
					return 0;
				}
				else if(strncasecmp(tag,"channel",7)==0){
//					mprintf("\t\t\tTAG=>[%s]\n",tag);
					tmp_p=strtok(tag,"channel");
//					mprintf("\t\t\ttmp_p=>[%s]\n",tmp_p);
					tmp=atoi(tmp_p);
//					mprintf("\t\t\ttmp=>[%d]\n",tmp);
//					mprintf("\t\t\tvalue=>[%s]\n",value);
					conf_p->OpenPri.card[cardno].iface[tmp].channel=tmp;
					strcpy((char *)conf_p->OpenPri.card[cardno].iface[tmp].name,value);
					strcpy((char *)conf_p->OpenPri.card[cardno].iface[tmp].card,(char *)conf_p->OpenPri.card[cardno].name);
//					mprintf("\t\tName=>[%s]\n",conf_p->OpenPri.iface[tmp].name);
					return 0;
				}
			}
		break;

		case C_SUBSECT:
		break;
	}
	return -1;
}

int conf_get_tag_value(char * tag,char * value,char * buffer)
{
	int got_tag=0;
	char * buf_p=buffer;
	char * tag_p=tag;
	char * val_p=value;
	while(*buf_p != '\0'){
		if (*buf_p == '='){
			got_tag=1;
		}
		else {
			if(!got_tag){
				*tag_p=*buf_p;
				tag_p++;
			}
			else {
				*val_p=*buf_p;
				val_p++;
			}
		}
		buf_p++;
	}
	if(!got_tag){
		*tag='\0';
		*value='\0';
		return -1;
	}
	else {
		*tag_p='\0';
		*val_p='\0';
		return 0;
	}
}

void conf_save_tone(CONF_INFO * conf_p, char * tone)
{
	return;
}

#if 0
void conf_dumper(CONF_INFO * conf_p)
{
	int file;
	int size;
	file = open("conf_dump",O_CREAT|O_WRONLY|O_TRUNC);
	size = write(file,conf_p,sizeof(CONF_INFO));
	close(file);
	mprintf("Wrote to conf_dump [%d]\n",size);
	return;
}
#endif

void conf_dump_print(CONF_INFO * conf_p)
{
	int i,x,z;
	mprintf("=================== GENERAL ========================\n");
	mprintf("Country code [%d]\n",conf_p->country_code);
	mprintf("Tone count[%d]\n",conf_p->tone_count);
	if (conf_p->tone_count > 0){
		for (i=0;i<conf_p->tone_count;i++){
			mprintf("\t Tone [%d]:\n",i);
			mprintf("\t\t Defn [%s]\n",conf_p->tones[i].defn);
			mprintf("\t\t Type [%s]\n",conf_p->tones[i].type);
			mprintf("\t\t Freq1 [%d]\n",conf_p->tones[i].freq1);
			mprintf("\t\t Band1 [%d]\n",conf_p->tones[i].band1);
			mprintf("\t\t Time On [%d]\n",conf_p->tones[i].ton);
			mprintf("\t\t Time Off [%d]\n",conf_p->tones[i].toff);
			mprintf("\t\t Freq2 [%d]\n",conf_p->tones[i].freq2);
			mprintf("\t\t Band2 [%d]\n",conf_p->tones[i].band2);
			mprintf("\t\t snr [%d]\n",conf_p->tones[i].snr);
			mprintf("\t\t twist [%d]\n",conf_p->tones[i].twist);
		}
	}
	mprintf("Card Count[%d]\n",conf_p->cardcount);
	if (conf_p->cardcount > 0 ){
		for (i=0;(i<conf_p->cardcount)&&(i<254);i++){
			mprintf("\tCard[%d] type[%d] number[%d]\n",i,conf_p->cards[i].type,conf_p->cards[i].number);
		}
	}
	mprintf("=================== OpenPRI ========================\n");
	mprintf("Channels [%d]\n",conf_p->OpenPri.channels);
	mprintf("Cards [%d]\n",conf_p->OpenPri.cards);
	for (x=0;x<conf_p->OpenPri.cards;x++){
		mprintf("\tCard Num [%d]\n",x);
		mprintf("\tInterface type [%d]\n",conf_p->OpenPri.card[x].if_type);
		mprintf("\tNode type [%d]\n",conf_p->OpenPri.card[x].node);
		mprintf("\tD-channel [%d]\n",conf_p->OpenPri.card[x].d_channel);
		z=(conf_p->OpenPri.card[x].if_type == VPB_PRI_E1?31:24);
		for (i=1;i<=z;i++){
			mprintf("\t Channel Num[%d]\n",i);
			mprintf("\t\t Channel [%d]\n",conf_p->OpenPri.card[x].iface[i].channel);
			mprintf("\t\t Channel Name [%s]\n",conf_p->OpenPri.card[x].iface[i].name);
			mprintf("\t\t Card Name [%s]\n",conf_p->OpenPri.card[x].iface[i].card);
			mprintf("\t\t Encoding [%s]\n",(VPB_ALAW==conf_p->OpenPri.card[x].iface[i].encoding?"alaw":(VPB_MULAW==conf_p->OpenPri.card[x].iface[i].encoding?"ulaw":"raw")));
		}
	}
	mprintf("=================== OpenLine =======================\n");
	mprintf("Channels [%d]\n",conf_p->OpenLine.channels);
	mprintf("Cards [%d]\n",conf_p->OpenLine.cards);
	for (x=0;x<conf_p->OpenLine.cards;x++){
		mprintf("\tCard Num [%d]\n",conf_p->OpenLine.card[x].cardnum);
		mprintf("\tCard type [%d]\n",conf_p->OpenLine.card[x].type);
		mprintf("\tCard channels [%d]\n",conf_p->OpenLine.card[x].channels);
		for(i=0;i<conf_p->OpenLine.card[x].channels;i++){
			mprintf("\t\tChannel [%d]\n",conf_p->OpenLine.card[x].iface[i].channel);
			mprintf("\t\tType [%d]\n",conf_p->OpenLine.card[x].iface[i].type);
			mprintf("\t\tSettings [%d]\n",conf_p->OpenLine.card[x].iface[i].settings);
			if (conf_p->OpenLine.card[x].iface[i].settings & MOD_HWPG){
				mprintf("\t\t\tHW Play Gain [%f]\n",conf_p->OpenLine.card[x].iface[i].hw_play_gain);
			}
			if (conf_p->OpenLine.card[x].iface[i].settings & MOD_HWRG){
				mprintf("\t\t\tHW Record Gain [%f]\n",conf_p->OpenLine.card[x].iface[i].hw_rec_gain);
			}
			if (conf_p->OpenLine.card[x].iface[i].settings & MOD_PG){
				mprintf("\t\t\tPlay Gain [%f]\n",conf_p->OpenLine.card[x].iface[i].play_gain);
			}
			if (conf_p->OpenLine.card[x].iface[i].settings & MOD_RG){
				mprintf("\t\t\tRecord Gain [%f]\n",conf_p->OpenLine.card[x].iface[i].rec_gain);
			}
			if (conf_p->OpenLine.card[x].iface[i].settings & MOD_BALS){
				mprintf("\t\t\tBal1[0x%x]\n",conf_p->OpenLine.card[x].iface[i].codec_hyb_bal1);
				mprintf("\t\t\tBal2[0x%x]\n",conf_p->OpenLine.card[x].iface[i].codec_hyb_bal2);
				mprintf("\t\t\tBal3[0x%x]\n",conf_p->OpenLine.card[x].iface[i].codec_hyb_bal3);
			}
		}
	}
	mprintf("=================== OpenSwitch =======================\n");
	mprintf("Channels [%d]\n",conf_p->OpenSwitch.channels);
	mprintf("Cards [%d]\n",conf_p->OpenSwitch.cards);
	for (x=0;x<conf_p->OpenSwitch.cards;x++){
		mprintf("\tCard Num [%d]\n",conf_p->OpenSwitch.card[x].cardnum);
		mprintf("\tCard type [%d]\n",conf_p->OpenSwitch.card[x].type);
		mprintf("\tCard channels [%d]\n",conf_p->OpenSwitch.card[x].channels);
		for(i=0;i<conf_p->OpenSwitch.card[x].channels;i++){
			mprintf("\t\tChannel [%d]\n",conf_p->OpenSwitch.card[x].iface[i].channel);
			mprintf("\t\tType [%d]\n",conf_p->OpenSwitch.card[x].iface[i].type);
			mprintf("\t\tSettings [%d]\n",conf_p->OpenSwitch.card[x].iface[i].settings);
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_HWPG){
				mprintf("\t\t\tHW Play Gain [%f]\n",conf_p->OpenSwitch.card[x].iface[i].hw_play_gain);
			}
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_HWRG){
				mprintf("\t\t\tHW Record Gain [%f]\n",conf_p->OpenSwitch.card[x].iface[i].hw_rec_gain);
			}
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_PG){
				mprintf("\t\t\tPlay Gain [%f]\n",conf_p->OpenSwitch.card[x].iface[i].play_gain);
			}
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_RG){
				mprintf("\t\t\tRecord Gain [%f]\n",conf_p->OpenSwitch.card[x].iface[i].rec_gain);
			}
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_BALS){
				mprintf("\t\t\tBal1[0x%x]\n",conf_p->OpenSwitch.card[x].iface[i].codec_hyb_bal1);
				mprintf("\t\t\tBal2[0x%x]\n",conf_p->OpenSwitch.card[x].iface[i].codec_hyb_bal2);
				mprintf("\t\t\tBal3[0x%x]\n",conf_p->OpenSwitch.card[x].iface[i].codec_hyb_bal3);
			}
		}
	}
    // This is handled by the vtcore layer now.
    #if 0
	mprintf("=================== OpenPCI =======================\n");
	mprintf("Channels [%d]\n",conf_p->OpenPci.channels);
	mprintf("Cards [%d]\n",conf_p->OpenPci.cards);
	for (x=0;x<conf_p->OpenPci.cards;x++){
		mprintf("\tCard Num [%d]\n",conf_p->OpenPci.card[x].cardnum);
		mprintf("\tCard type [%d]\n",conf_p->OpenPci.card[x].type);
		mprintf("\tCard channels [%d]\n",conf_p->OpenPci.card[x].channels);
/*		for(i=0;i<conf_p->OpenSwitch.card[x].channels;i++){
			mprintf("\t\tChannel [%d]\n",conf_p->OpenSwitch.card[x].iface[i].channel);
			mprintf("\t\tType [%d]\n",conf_p->OpenSwitch.card[x].iface[i].type);
			mprintf("\t\tSettings [%d]\n",conf_p->OpenSwitch.card[x].iface[i].settings);
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_HWPG){
				mprintf("\t\t\tHW Play Gain [%f]\n",conf_p->OpenSwitch.card[x].iface[i].hw_play_gain);
			}
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_HWRG){
				mprintf("\t\t\tHW Record Gain [%f]\n",conf_p->OpenSwitch.card[x].iface[i].hw_rec_gain);
			}
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_PG){
				mprintf("\t\t\tPlay Gain [%f]\n",conf_p->OpenSwitch.card[x].iface[i].play_gain);
			}
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_RG){
				mprintf("\t\t\tRecord Gain [%f]\n",conf_p->OpenSwitch.card[x].iface[i].rec_gain);
			}
			if (conf_p->OpenSwitch.card[x].iface[i].settings & MOD_BALS){
				mprintf("\t\t\tBal1[0x%x]\n",conf_p->OpenSwitch.card[x].iface[i].codec_hyb_bal1);
				mprintf("\t\t\tBal2[0x%x]\n",conf_p->OpenSwitch.card[x].iface[i].codec_hyb_bal2);
				mprintf("\t\t\tBal3[0x%x]\n",conf_p->OpenSwitch.card[x].iface[i].codec_hyb_bal3);
			}
		} */
	}
    #endif
}

/*--------------------------------------------------------------------------*\

	FUNCTION....: logger
	AUTHOR......: Ben Kramer
	DATE CREATED: 02/08/2004

\*--------------------------------------------------------------------------*/
/*
void logger (const char* const log_name, char *fmt, ...)
{
	char    s[1024];
	va_list argptr;
	time_t  curtime;
	struct  tm *loctime;
	char    temp[20];
	char	tstamp[9];
	char	*tmp;
	int	rc;
	struct	stat finfo;

	// this produces the output from the varying input
	va_start(argptr, fmt);
	vsnprintf(s,(1024 - 30), fmt, argptr);
	va_end(argptr);
	
	// This removes any cariage returns
	tmp=strstr(s,"\n");
	if(tmp !=NULL){
		*tmp=0;
	}

	curtime = time (NULL);
	loctime = localtime (&curtime);
	sprintf(temp,"%04d/%02d/%02d-%02d:%02d:%02d", 
		loctime->tm_year+1900,
		loctime->tm_mon+1,
		loctime->tm_mday,
		loctime->tm_hour,
		loctime->tm_min,
		loctime->tm_sec);
	
	sprintf(tstamp, "%04d%02d%02d",
		loctime->tm_year+1900,
		loctime->tm_mon+1,
		loctime->tm_mday
		);

	if (logfileh){
		rc = fstat(fileno(logfileh),&finfo);
		if (rc == 0 ){
			if (finfo.st_size > MAX_LOG_BYTES){
				fclose(logfileh);
				logfileh=NULL;
			}
			else {
				if (strcmp(logstamp,tstamp) != 0 ){ // roll the log file!
					fprintf(logfileh,"%s Whirr Clunk! Rolling log file...\n",temp);
					rolllogger(logfilepath,log_name);
				}
				fprintf(logfileh,"%s %s \n",temp,s);
			}
		}
	}
	else if (strcmp(logstamp,tstamp) != 0 ){ // roll the log file!
		fprintf(logfileh,"%s Whirr Clunk! Rolling log file...\n",temp);
		rolllogger(logfilepath,log_name);
		fprintf(logfileh,"%s %s \n",temp,s);
	}
	else {
		mprintf("%s %s \n",temp,s);
	}
	return;
}
*/
/*--------------------------------------------------------------------------*\

	FUNCTION....: openlogger
	AUTHOR......: Ben Kramer
	DATE CREATED: 02/08/2004

\*--------------------------------------------------------------------------*/
/*
int openlogger(const char* const dir,const char* const log_name)
{
	time_t  curtime;
	struct  tm *loctime;

	curtime = time (NULL);
	loctime = localtime (&curtime);
	sprintf(logfilename, "%s/%s-%04d%02d%02d.log", dir, log_name,
		loctime->tm_year+1900,
		loctime->tm_mon+1,
		loctime->tm_mday
	);
	sprintf(logstamp, "%04d%02d%02d",
		loctime->tm_year+1900,
		loctime->tm_mon+1,
		loctime->tm_mday
	);
	logfileh=fopen(logfilename,"a");
	if(logfileh){
		setvbuf(logfileh, (char *)NULL, _IOLBF, 0);
		logger(log_name,"Whirr Click! Log opened");
		return(0);
	}
	else {
		return(-1);
	}
}
*/
/*--------------------------------------------------------------------------*\

	FUNCTION....: closelogger
	AUTHOR......: Ben Kramer
	DATE CREATED: 03/08/2004

\*--------------------------------------------------------------------------*/
/*
int closelogger(const char* const log_name)
{
	logger(log_name,"Closing log file....");
	fclose(logfileh);
	logfileh=NULL;
	return 0;
}
*/
/*--------------------------------------------------------------------------*\

	FUNCTION....: rolllogger
	AUTHOR......: Ben Kramer
	DATE CREATED: 02/08/2004


\*--------------------------------------------------------------------------*/
/*
void rolllogger(const char* const dir, const char* const log_name)
{
	char    s[256];
	time_t  curtime;
	struct  tm *loctime;
	int	rc;

	if (logfileh){
		rc=fclose(logfileh);
	}
	curtime = time (NULL);
	loctime = localtime (&curtime);
	sprintf(s, "%s/%s-%04d%02d%02d.log", dir, log_name,
		loctime->tm_year+1900,
		loctime->tm_mon+1,
		loctime->tm_mday);
	sprintf(logstamp, "%04d%02d%02d",
		loctime->tm_year+1900,
		loctime->tm_mon+1,
		loctime->tm_mday
		);
	logfileh=fopen(s,"a");
	if (logfileh){
		setvbuf(logfileh, (char *)NULL, _IOLBF, 0);
		logger(log_name,"Whirr Click! Rolling Log - Opened");
	}
}
*/
